package com.blackledge.david.html.hme;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.regex.Pattern;

import com.blackledge.david.html.hme.WebBrowserApp.MEDIA;
import com.blackledge.david.tivo.V49BApplication;
import com.tivo.hme.interfaces.IContext;

public class WebBrowserOptions {

	/** Whether User-Agent includes AvantGo so websites think we're mobile */
	private boolean appearMobile = true;
	/** current scale to use when rendering fonts */
	private float fontScale = 1.0f;
	private boolean leftSameAsBack = true;
	private boolean maximizedDisplay = false;
	private boolean shaded = false;
	private boolean showTV = false;
	private MEDIA activeMediaStyle = MEDIA.TV;
		
	private WebBrowserApp browser;
	private String showMediaURL;
	private static final String OPTIONS_KEY = "OPTIONS"; //$NON-NLS-1$
	public static final String URL_FONTSCALE = "fontscale";
	public static final String URL_THEME = "theme";
	public static final String URL_APPMODE = "disablebookmarks";
	public static final String URL_MOBILE = "mobile";
	public static final String URL_STYLE = "style";
	public static final String URL_TABLES = "tables";
	public static final String URL_MEDIA = "media";

	private enum OPTIONS {
		MOBILE, SHOW_TV, MEDIA_STYLE, FONT_SCALE, MAXIMIZED, LEFT_BACK, SHADED
	}

	public WebBrowserOptions(WebBrowserApp webBrowser) {
		this.browser = webBrowser;
	}

	private IContext getContext() {
		return this.browser.getContext();
	}

	boolean isMediaStyleActive(MEDIA type) {
		return type.equals(this.activeMediaStyle);
	}

	boolean isAppearingMobile() {
		return this.appearMobile;
	}

	boolean isLeftSameAsBack() {
		return this.leftSameAsBack;
	}

	boolean isMaximized() {
		return this.maximizedDisplay;
	}
	
	boolean isShaded() {
		return this.shaded;
	}

	boolean isShowingLiveTV() {
		if(this.showTV) {
			return this.showMediaURL == null
					|| this.showMediaURL.equals(V49BApplication.STREAM_LIVETV);
		} else {
			return false;
		}
	}
	
	String getMediaURL() {
		if (isShowingLiveTV()) {
			return V49BApplication.STREAM_LIVETV;
		} else if (this.showTV) {
			return this.showMediaURL;
		}
		return null;
	}
	
	void loadOptions() {
		String optionsString = getContext().getPersistentData(OPTIONS_KEY);
		loadOptions(optionsString);
	}
	
	void loadOptions(String optionsString) {
		if (optionsString == null)
			return;
		WebBrowserApp.debugOut("LOAD OPTIONS: "+optionsString); //$NON-NLS-1$
		String[] options = optionsString.split(","); //$NON-NLS-1$
		if (options.length > OPTIONS.MOBILE.ordinal())
			this.appearMobile = Boolean.parseBoolean(options[OPTIONS.MOBILE.ordinal()]);
		if (options.length > OPTIONS.SHOW_TV.ordinal()) {
			this.showMediaURL = null;
			String opt = options[OPTIONS.SHOW_TV.ordinal()];
			if (opt.indexOf(':') >= 0) {
				this.showTV = true;
				this.showMediaURL = opt;
			} else {
				this.showTV = Boolean.parseBoolean(opt);
			}
		}
		if (options.length > OPTIONS.MEDIA_STYLE.ordinal() && options[OPTIONS.MEDIA_STYLE.ordinal()].trim().length() > 0)
			this.activeMediaStyle = MEDIA.valueOf(options[OPTIONS.MEDIA_STYLE.ordinal()]);
		if(options.length > OPTIONS.FONT_SCALE.ordinal())
			this.fontScale = Float.parseFloat(options[OPTIONS.FONT_SCALE.ordinal()]);
		if (options.length > OPTIONS.MAXIMIZED.ordinal())
			this.maximizedDisplay = Boolean.parseBoolean(options[OPTIONS.MAXIMIZED.ordinal()]);
		if (options.length > OPTIONS.LEFT_BACK.ordinal())
			this.leftSameAsBack = Boolean.parseBoolean(options[OPTIONS.LEFT_BACK.ordinal()]);
		if (options.length > OPTIONS.SHADED.ordinal())
			this.shaded = Boolean.parseBoolean(options[OPTIONS.SHADED.ordinal()]);
	}

	void saveOptions() {
		String options = getOptionsString();
		getContext().setPersistentData(OPTIONS_KEY, options);
	}

	String getOptionsString() {
		StringBuffer optionsString = new StringBuffer();
		optionsString.append(this.appearMobile).append(","); // 0 //$NON-NLS-1$
		optionsString.append(escapeCommas(this.showMediaURL)).append(","); // 1 //$NON-NLS-1$
		optionsString.append(this.activeMediaStyle).append(","); // 2 //$NON-NLS-1$
		optionsString.append(this.fontScale).append(","); // 3 //$NON-NLS-1$
		optionsString.append(this.maximizedDisplay).append(","); //4 //$NON-NLS-1$
		optionsString.append(this.leftSameAsBack).append(","); //5 //$NON-NLS-1$
		optionsString.append(this.shaded).append(","); //6 //$NON-NLS-1$
		
		return optionsString.toString();
	}

	private String escapeCommas(String url) {
		if(url == null) return "null";
		return url.replaceAll(Pattern.quote(","), "%2C");
	}

	/**
	 * triggers {@link #saveOptions()}
	 */
	void setMediaStyleActive(MEDIA type) {
		this.activeMediaStyle = type;
		saveOptions();
	}
	
	float getFontScale() {
		return this.fontScale;
		// don't need to re-read it... it was read, and is saved every time it is set, so this is no different.
//		String optionsString = getContext().getPersistentData(OPTIONS_KEY);
//		if (optionsString == null)
//			return 0f;
//		String[] options = optionsString.split(","); //$NON-NLS-1$
//		if(options.length >= OPTIONS.FONT_SCALE.ordinal())
//			return Float.parseFloat(options[OPTIONS.FONT_SCALE.ordinal()]);
//		return 0f;
	}

	/**
	 * triggers {@link #saveOptions()}
	 */
	void setAppearingMobile(boolean b) {
		this.appearMobile = b;
	
		saveOptions();
	}

	void setFontScale(float fontScale) {
		this.fontScale = fontScale;
		saveOptions();
	}

	void setLeftSameAsBack(boolean leftSameAsBack) {
		this.leftSameAsBack = leftSameAsBack;
		saveOptions();
	}

	/**
	 * triggers {@link #saveOptions()}
	 */
	void setMaximized(boolean b) {
		this.maximizedDisplay = b;
	
		saveOptions();
	}

	public void setShaded(boolean shade) {
		this.shaded = shade;
		
		saveOptions();
	}

	/**
	 * triggers {@link #saveOptions()}
	 */
	void setShowLiveTV(boolean showTV) {
		this.showTV = showTV;
		if(this.showTV) {
			this.showMediaURL = null;
		}
	
		saveOptions();
	}
	
	public void setShowMedia(String media) {
		this.showTV = (media != null);
		this.showMediaURL = media;
		
		saveOptions();
	}

	public String getURLOptions() {
		StringBuffer o = new StringBuffer();
		o.append(URL_MOBILE).append("=").append(appearMobile).append("&");
		o.append(URL_FONTSCALE).append("=").append(this.fontScale).append("&");
		o.append(URL_STYLE).append("=").append(this.activeMediaStyle.name()).append("&");
		if(this.showTV) {
			o.append(URL_MEDIA).append("=").append(urlEscape(V49BApplication.STREAM_LIVETV)).append("&");
		}
//		this.leftSameAsBack;
//		this.maximizedDisplay;
//		o.append(URL_APPMODE)
//		o.append(URL_TABLES)
//		o.append(URL_THEME)
		return o.toString();
	}

	private String urlEscape(String str) {
		try {
			return URLEncoder.encode(str, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			return str;
		}
	}

	/** @deprecated not complete */
	public void loadURLOptions(String stringURL) {
		URL url;
		try {
			url = new URL(stringURL);
		} catch (MalformedURLException e) {
			// FIXME Auto-generated catch block
			e.printStackTrace();
			return;
		}
		url.getQuery();
		//FIXME etc... if this is useful
	}

}

