package com.blackledge.david.tivo.widgets;

import java.awt.Color;

import com.tivo.hme.bananas.BEvent;
import com.tivo.hme.bananas.BList;
import com.tivo.hme.bananas.BView;

import com.tivo.hme.sdk.HmeEvent;
import com.tivo.hme.sdk.View;

/**
 * BList with a row number displayed which supports entering a new number for
 * reordering the list (similar to Series 4 Season Pass management). fires
 * {@link #ACTION_LIST_ORDER_CHANGE} after number is entered, during which
 * {@link #getRequestedPosition()} has the new value. Numbering begins after
 * {@link #setPositionOffset(int)}.
 */
public class BNumberedList extends BList {
	public static final String ACTION_LIST_ORDER_CHANGE = "listOrderChange";
	
	private int positionOffset;
	private boolean enteringNumber = false;
	private String number = null;
	private BView numberView = null;

	public BNumberedList(BView parent, int x, int y,
			int width, int height, int rowHeight) {
		super(parent, x, y, width, height, rowHeight);
	}

	/**
	 * Handle focus movement. (override just fixes a rare bug in BList)
	 */
	public boolean handleFocus(boolean isGained, BView gained, BView lost) {
		// work around a bug in BList when gained is null.
		if (!isGained && gained == null) {
			refresh();
			return true;
		} else {
			return super.handleFocus(isGained, gained, lost);
		}
	}
	
	public void setPositionOffset(int offset) {
		this.positionOffset = offset;
	}
	
	public int getPositionOffset() {
		return this.positionOffset;
	}

	/**
	 * Overridden to include the row number in the row's view. Call this super
	 * implementation at the beginning of your row implementation in order to
	 * show the row number (relative to PositionOffset)
	 */
	@Override
	protected void createRow(BView parent, int index) {
		if (index >= positionOffset) {
			BView numberView = createRowNumberView(parent);
			setRowNumber(numberView, String.valueOf(index - positionOffset  + 1), false);
			numberView.setTransparency(0.5f);
		}
	}

	/**
	 * Overridden to catch position number entry KEY events ahead of time. If
	 * number entry is in progress, prevents normal processing of related keys.
	 * Unrelated keys cancel number entry and are passed through.
	 * 
	 * @param event
	 * @return
	 */
	public boolean handleEvent(HmeEvent event) {
		synchronized (lock) {
			if (enteringNumber) {
				if (event.getOpCode() == EVT_KEY) {
					HmeEvent.Key ir = (HmeEvent.Key) event;
					if (ir.getAction() == KEY_PRESS) {
						switch (ir.getCode()) {
						case KEY_NUM0:
						case KEY_NUM1:
						case KEY_NUM2:
						case KEY_NUM3:
						case KEY_NUM4:
						case KEY_NUM5:
						case KEY_NUM6:
						case KEY_NUM7:
						case KEY_NUM8:
						case KEY_NUM9:
							if (!numberEnter(ir.getCode() - KEY_NUM0)) {
								getBApp().play("bonk.snd");
							} else {
								getBApp().play("updown.snd");
							}
							return true;
						case KEY_REVERSE:
							if (!numberBackspace()) {
								getBApp().play("bonk.snd");
							}
							return true;
						case KEY_FORWARD:
						case KEY_SELECT:
						case KEY_ENTER:
							getBApp().play("select.snd");
							numberEntered();
							// always return true... numberEntered ends the mode, but shouldn't forward the event.
							return true;
						default:
							// everything else cancels number mode. and then is
							// passed through
							numberCancel();
							if (enteringNumber)
								return true;
							break;
						}
					}
				}
			} else {
				if (event.getOpCode() == EVT_KEY) {
					HmeEvent.Key ir = (HmeEvent.Key) event;
					if (ir.getAction() == KEY_PRESS) {
						switch (ir.getCode()) {
						case KEY_NUM0:
						case KEY_NUM1:
						case KEY_NUM2:
						case KEY_NUM3:
						case KEY_NUM4:
						case KEY_NUM5:
						case KEY_NUM6:
						case KEY_NUM7:
						case KEY_NUM8:
						case KEY_NUM9:
							if (!numberEnter(ir.getCode() - KEY_NUM0)) {
								getBApp().play("bonk.snd");
							} else {
								getBApp().play("updown.snd");
							}
							// we just started taking over.
							if (enteringNumber)
								return true;
							break;
						}
					}
				}
			}
			return super.handleEvent(event);
		} // synch
	} // handleEvent()
	
	private void numberEntered() {
		postEvent(new BEvent.Action(this, ACTION_LIST_ORDER_CHANGE));
		numberCancel();
	}
	
	/**
	 * Get the numeric value entered by the user. Use when a
	 * {@link #ACTION_LIST_ORDER_CHANGE} action event is fired by this list.
	 * Value is the new position (index-positionOffset+1) the user wants for the
	 * currently focused list item. Only valid during that event.
	 * 
	 * @return Numeric position (index-positionOffset+1) entered by user, or -1
	 *         if no valid value is currently entered.
	 */
	public int getRequestedPosition() {
		if (number == null) {
			return -1;
		} else {
			try {
				return Integer.parseInt(number);
			} catch (Exception e) {
				return -1;
			}
		}
	}

	private void showEnteredNumber() {
		showNumber(this.number, true);
	}
	
	private void showNumber(String number, boolean editing) {
		if(number == null) {
			if(numberView != null) {
				View textView = numberView.getChild(0);
				if(textView.getResource() != null)
					textView.getResource().remove();
				if(textView != null)
					textView.remove();
				// don't actually remove the resource we're using for the background... we need to reuse it.
				numberView.remove();
				numberView = null;
			}
		} else {
			if(numberView == null && getFocus() >= 0) {
				BView rowView;
				try {
					rowView = getRow(getFocus());
				} catch (ArrayIndexOutOfBoundsException e) {
					return;
				}
				if (rowView == null)
					return;
				
				numberView = createRowNumberView(rowView);
				numberView.setTransparency(1.0f);
				numberView.setTransparency(0.0f, getResource(ANIM));
			}
			if (numberView != null) {
				setRowNumber(numberView, number, editing);
			}
		}
	}

	private void setRowNumber(BView numberView, String number, boolean editing) {
		View textView = numberView.getChild(0);
		if(textView.getResource() != null)
			textView.getResource().remove();
		Object font = "default-" + getFontSizeForView(numberView.getHeight()) + "-italic.font";
		Color color = Color.white;
		String text = number;
		if (editing) {
			font = "default-" + getFontSizeForView(numberView.getHeight()) + ".font";
			text = "  "+number + "|";
		}
		textView.setResource(textView.createText(font, color, text));
	}

	private int getFontSizeForView(int viewHeight) {
		int size = viewHeight - 7;
		if(size < 4) {
			return 4;
		} else {
			return size;
		}
	}
	
	@Override
	public void setFocus(int index, boolean animate) {
		super.setFocus(index, animate);

		showNumber(null, false);
		if (index >= positionOffset) {
			String num = String.valueOf(index - positionOffset  + 1);
			showNumber(num, false);
		}
	}
	
	private BView createRowNumberView(BView rowView) {
		int viewHeight = rowView.getHeight()-2;
		int numwidth = 3 * getFontSizeForView(viewHeight);
		BView view = new BView(rowView, rowView.getWidth()-numwidth, 1, numwidth, viewHeight);
		Object resource = null;
		try {
			resource = getBApp().getSkin().get("numbered-list-field").getResource();
		} catch (Exception e) {
			try {
				resource = getBApp().getSkin().get("keyboard-focus").getResource();
			} catch (Exception e2) {
				resource = null;
			}
		}
		if(resource == null) {
			resource = "com/tivo/hme/bananas/keyboard-datafield.png";
//			resource = Color.blue;
		}
		view.setResource(resource, RSRC_HALIGN_CENTER | RSRC_VALIGN_CENTER | RSRC_IMAGE_VFIT);
		// the text holder:
		new BView(view, 0, 0, numwidth, view.getHeight());
		return view;
	}
	
	private void numberCancel() {
		boolean wasEnteringNumber = enteringNumber;
		enteringNumber = false;
		number = null;
		if (wasEnteringNumber) {
			showEnteredNumber();
		}
	}
	
	private boolean numberBackspace() {
		if(number == null || number.length() == 0) {
			return false;
		}
		
		number = number.substring(0, number.length()-1);
		
		if(number.length() == 0) {
			numberCancel();
		} else {
			showEnteredNumber();
		}
		return true;
	}
	
	private boolean numberEnter(int num) {
		// max of 1000 positions
		if(number != null && number.length() >= 3) {
			return false;
		}
		enteringNumber = true;
		if(number == null) {
			number = String.valueOf(num);
		} else {
			number += String.valueOf(num);
		}
		// could return false if number is higher than list, but just go to bottom of list in that case, instead
		showEnteredNumber();
		return true;
	}

}
