/**
 * $Header: /CVSSTORAGE/repository1/TiVo/TyPo for TiVo/com/blackledge/david/tivo/widgets/typo/WordList.java,v 1.2 2009/12/30 02:26:28 dlblack Exp $
 */
package com.blackledge.david.tivo.widgets.typo;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * Rapid access to top 10 suggestions for a given starting string.
 *
 * @version $Revision: 1.2 $
 */
public class WordList {
    
        //Singleton pattern.
        private static WordList instance = null;
        public synchronized static WordList getInstance() {
            if(instance == null) {
                instance = new WordList();
            }
            return instance;
        }
        private WordList() {
        	init();
        }
            
        private ArrayList<String> list;
        private WordList.WordStruct topStruct;
        private boolean skipLinesWithSpaces = true;

        public boolean isSkipLinesWithSpaces() {
            return this.skipLinesWithSpaces;
        }
    
        /**
         * Change behavior of "load" methods to allow entries with spaces in them.
         * Note: setting this to false may include unexpected entries such as
         * opening documentation lines in the file.  "addWord" does not obey this setting.
         * @param skipLinesWithSpaces (true by default)
         */
        public void setSkipLinesWithSpaces(boolean skipLinesWithSpaces) {
            this.skipLinesWithSpaces = skipLinesWithSpaces;
        }
        
        /**
         * Adds another file's contents to the word list. All blank lines and
         * lines containing spaces are ignored.
         * @param file
         * @throws IOException
         */
        public void load(File file) throws IOException {
            InputStream inputStream = new FileInputStream( file );
            load( inputStream );
        }

    /**
     * Adds another inputStream's contents to the word list. All blank lines and
     * lines containing spaces are ignored. Although duplicates will not be
     * shown in suggestions, they are not prevented in the main word list.
     * @param inputStream
     * @throws IOException
     */
        public void load(InputStream inputStream)
            throws IOException
        {
            if(topStruct == null)
                init();
            BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
            for(String line = reader.readLine() ; line != null ; line = reader.readLine()) {
                if(line.trim().length() == 0 || (skipLinesWithSpaces && line.indexOf( " " ) > -1))
                    continue;
//                if(!list.contains(line.trim())) { // waaaaaay too slow
                list.add(line.trim());
            }

            for(int i = 0 ; i < list.size() ; ++i) {
                topStruct.addNextWord(i);
            }
        }

        private void init() {
            topStruct = new WordStruct(0);
            list = new ArrayList<String>();
        }

    /**
     * @param word the word-so-far
     * @return a short list (no more than 10 entries) of words that start with
     *         the word-so-far
     */
        public List<String> getSuggestions(String word) {
            if(topStruct == null) loadDefault();
            return topStruct.getSuggestion(word);
        }
        /**
         * 
         * @param word the word-so-far
         * @return a list of suggested next letters based on the current word list.
         */
        public List<Character> getSuggestionLetters(String word) {
            if(topStruct == null) loadDefault();
            return topStruct.getSuggestionLetters(word, getSuggestions(word));
        }

        /**
         * @param index must be less than {@link #getWordCount()}
         * @return the word loaded into the given index.
         */
        public String getWord(int index) {
            if(topStruct == null) loadDefault();
            return list.get(index);
        }
        
        /**
         * Add an arbitrary word or phrase to the word list.
         * @param word
         */
        public void addWord(String word) {
            list.add(word);
            topStruct.addNextWord( list.size()-1 );
        }
        
        /**
         * @return the number of words in the list.
         */
        public int getWordCount() {
            if(topStruct == null) loadDefault();
            return list.size();
        }

        /**
         * Keeps a list of the top 10 words that are the same up through the
         * passed-in index, and keeps a map of ALL words larger than that with
         * the same start based on the next character, and another one of these
         * structures.
         * @version $Revision: 1.2 $
         */
        private static class WordStruct {
            private static int SUGGESTION_LIST_SIZE = 10;
            private int indexSoFar; 
            private int top10[] = new int[SUGGESTION_LIST_SIZE];
            private int top10Length = 0;
            private HashMap<Character, WordList.WordStruct> mapNextLetter = null;

            WordStruct(int indexSoFar) {
                this.indexSoFar = indexSoFar;
            }

            /**
             * @param word WordList index to a word that shares the first
             *        indexSoFar letter with all other word indexes that will be
             *        passed in to this instance.
             */
            void addNextWord(int word) {
                if(Arrays.asList(top10).contains(word)) return; // repeated
                if(top10Length < SUGGESTION_LIST_SIZE) { // first 10 additions are in the top 10
                    for(int i = 0 ; i < top10Length ; ++i) {
                        if(getWord(top10[i]).equals( getWord(word) )) {
                            return;
                        }
                    }
                    top10[top10Length] = word;
                    ++top10Length;
                    if(top10Length == 1) return; // defer creating map until needed. This also stops recursion
                    // TODO could go further...only create map if two words share a next char (or when all 10 slots are filled), otherwise just scan list.
                    else if(top10Length == 2) {
                        String wordString = getWord(top10[0]);
                        if(wordString.length() > indexSoFar)
                            mapAddNextWord(wordString.charAt(indexSoFar), top10[0], indexSoFar+1);
                    }
                }
                String wordString = getWord(word);
                if(wordString.length() > indexSoFar)
                    mapAddNextWord(wordString.charAt(indexSoFar), word, indexSoFar+1);
            }


            private String getWord(int wordIndex) {
                return WordList.getInstance().getWord(wordIndex);
            }


            private void mapAddNextWord(Character chr, int wordIndex, int nextIndexSoFar) {
                if(mapNextLetter == null) mapNextLetter = new HashMap<Character, WordList.WordStruct>();
                WordList.WordStruct struct = mapNextLetter.get(chr);
                if(struct == null) {
                    struct = new WordStruct(nextIndexSoFar);
                    mapNextLetter.put(chr, struct);
                }
                struct.addNextWord(wordIndex);
            }

            List<String> getSuggestion(String word) {
                if(word.length() == 0) {
                    // word exhausted, our list must be THE list.
                    ArrayList<String> result = new ArrayList<String>();
                    for(int i = 0 ; i < top10Length ; ++i) {
                        result.add(getWord(top10[i]));
                    }
                    return result;
                }
                if(top10Length==1) {
                    // no more words exist after this, our single word is the 
                    // only possible suggestion.
                    String topWord = getWord(top10[0]);
                    // but it's only a suggestion if it fits.
                    if(topWord.indexOf(word,indexSoFar)==indexSoFar) {
                        ArrayList<String> result = new ArrayList<String>();
                        result.add(topWord);
                        return result;
                    }
                    else return null;
                }
                // if no map, nothing else to look up
                if(mapNextLetter == null) return null;
                WordList.WordStruct next = mapNextLetter.get(word.charAt(0));
                // if nothing mapped to character, there is no known word that matches.
                if(next == null) return null;
                // recurse on the remainder of the string.
                return next.getSuggestion(word.substring(1));
            }

            List<Character> getSuggestionLetters(String word, List<String> words) {
                if(words == null) return null;
                ArrayList<Character> letters = new ArrayList<Character>();

                for(String verd : words) {
                    if(verd.length() > word.length()) {
                        char letter = verd.substring(word.length()).charAt(0);
                        if(! letters.contains(letter))
                            letters.add(letter);
                    }
                }

                return letters;
            }

        }

        public static void main(String[] args) throws IOException {
            WordList.getInstance().loadDefault();
            System.out.println(WordList.getInstance().getSuggestions( "Dav" ));
            System.out.println(WordList.getInstance().getSuggestions( "Black" ));
            System.out.println(WordList.getInstance().getSuggestions( "the" ));
            System.out.println(WordList.getInstance().getSuggestions( "Hi" ));
            System.out.println(WordList.getInstance().getSuggestionLetters( "Hi" ));
            System.out.println(WordList.getInstance().getSuggestions( "he" ));
            System.out.println(WordList.getInstance().getSuggestionLetters( "he" ));
            System.out.println(WordList.getInstance().getSuggestions( "hel" ));
            System.out.println(WordList.getInstance().getSuggestions( "x" ));
            System.out.println(WordList.getInstance().getSuggestions( "Xe" ));
            System.out.println(WordList.getInstance().getSuggestions( "xy" ));
            System.out.println(WordList.getInstance().getSuggestionLetters( "xy" ));
            System.out.println(WordList.getInstance().getSuggestions( "z" ));
            System.out.println(WordList.getInstance().getSuggestions( "" ));
        }

        private void loadDefault()
        {
            try {
                URL fileURL = WordList.class.getResource( "FREQ.TXT" );
                if(fileURL != null) {
                        load( new File( fileURL.toURI() ) );
                }
                fileURL = WordList.class.getResource( "COMMON.TXT" );
                if(fileURL != null) {
                        load( new File( fileURL.toURI() ) );
                }
            } catch(IOException e) {
                System.err.println("WordList loading Defaults: "+e);
                e.printStackTrace();
            } catch(URISyntaxException e) {
                System.err.println("WordList loading Defaults: "+e);
                e.printStackTrace();
            }
        }

}